@@ -3,7 +3,7 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Find the top repeated note phrases of a MIDI file
 
-our $VERSION = '0.0901';
+our $VERSION = '0.1000';
 
 use Moo;
 use strictures 2;
@@ -14,6 +14,7 @@ use Lingua::EN::Ngram;
 use List::Util qw( shuffle );
 use List::Util::WeightedChoice qw( choose_weighted );
 use MIDI::Simple;
+use MIDI::Util;
 use Music::Gestalt;
 use Music::Note;
 use Music::Tempo;
@@ -21,42 +22,42 @@ use Music::Tempo;
 
 has in_file => (
     is       => 'ro',
-    isa      => \&_invalid_list,
+    isa      => \&_is_list,
     required => 1,
 );
 
 
 has ngram_size => (
     is      => 'ro',
-    isa     => \&_invalid_integer,
+    isa     => \&_is_integer,
     default => sub { 2 },
 );
 
 
 has max_phrases => (
     is      => 'ro',
-    isa     => \&_invalid_integer,
+    isa     => \&_is_integer0,
     default => sub { 10 },
 );
 
 
 has bpm => (
     is      => 'ro',
-    isa     => \&_invalid_integer,
+    isa     => \&_is_integer,
     default => sub { 100 },
 );
 
 
 has durations => (
     is      => 'ro',
-    isa     => \&_invalid_list,
-    default => sub { [qw( qn tqn )] },
+    isa     => \&_is_list,
+    default => sub { [qw( hn qn en )] },
 );
 
 
 has patches => (
     is      => 'ro',
-    isa     => \&_invalid_list,
+    isa     => \&_is_list,
     default => sub { [ 0 .. 127 ] },
 );
 
@@ -76,55 +77,55 @@ has pause_duration => (
 
 has analyze => (
     is  => 'ro',
-    isa => \&_invalid_list,
+    isa => \&_is_list,
 );
 
 
 has loop => (
     is      => 'ro',
-    isa     => \&_invalid_integer,
+    isa     => \&_is_integer,
     default => sub { 10 },
 );
 
 
 has weight => (
     is      => 'ro',
-    isa     => \&_invalid_boolean,
+    isa     => \&_is_boolean,
     default => sub { 0 },
 );
 
 
 has random_patch => (
     is      => 'ro',
-    isa     => \&_invalid_boolean,
+    isa     => \&_is_boolean,
     default => sub { 0 },
 );
 
 
 has shuffle_phrases => (
     is      => 'ro',
-    isa     => \&_invalid_boolean,
+    isa     => \&_is_boolean,
     default => sub { 0 },
 );
 
 
 has single_phrases => (
     is      => 'ro',
-    isa     => \&_invalid_boolean,
+    isa     => \&_is_boolean,
     default => sub { 0 },
 );
 
 
 has one_channel => (
     is      => 'ro',
-    isa     => \&_invalid_boolean,
+    isa     => \&_is_boolean,
     default => sub { 0 },
 );
 
 
-has gestalt => (
+has bounds => (
     is      => 'ro',
-    isa     => \&_invalid_boolean,
+    isa     => \&_is_boolean,
     default => sub { 0 },
 );
 
@@ -224,7 +225,7 @@ sub process {
             }
 
             $analysis .= $self->_gestalt_analysis( \@events )
-                if $self->gestalt;
+                if $self->bounds;
         }
     }
 
@@ -246,17 +247,18 @@ sub _gestalt_analysis {
     $note    = Music::Note->new( $g->PitchMiddle, 'midinum' );
     my $mid  = $note->format('midi');
 
-    my $gestalt = "\tRange: $low to $high\n"
+    my $bounds = "\tRange: $low to $high\n"
         . "\tSpan: $mid +/- " . $g->PitchRange . "\n";
 
-    return $gestalt;
+    return $bounds;
 }
 
 
 sub populate {
     my ($self) = @_;
 
-    $self->score( _setup_midi( bpm => $self->bpm ) );
+    my $score = MIDI::Util::setup_score( bpm => $self->bpm );
+    $self->score($score);
 
     my @phrases;
     my $playback;
@@ -269,7 +271,7 @@ sub populate {
             my $func = sub {
                 my $patch = $self->random_patch ? $self->_random_patch() : 0;
 
-                _set_chan_patch( $self->score, $channel, $patch );
+                MIDI::Util::set_chan_patch( $self->score, $channel, $patch );
 
                 for my $n ( 1 .. $self->loop ) {
                     my $choice = choose_weighted(
@@ -332,7 +334,7 @@ sub populate {
             my $func = sub {
                 my $patch = $self->random_patch ? $self->_random_patch() : 0;
 
-                _set_chan_patch( $self->score, $channel, $patch);
+                MIDI::Util::set_chan_patch( $self->score, $channel, $patch);
 
                 for my $note ( @all ) {
                     if ( $note eq 'r' ) {
@@ -381,49 +383,22 @@ sub _convert {
     return $text;
 }
 
-sub _setup_midi {
-    my %args = (
-        volume  => 120,
-        bpm     => 100,
-        channel => 0,
-        patch   => 0,
-        octave  => 5,
-        @_,
-    );
-
-    my $score = MIDI::Simple->new_score();
-
-    $score->set_tempo( bpm_to_ms($args{bpm}) * 1000 );
-
-    $score->Volume($args{volume});
-    $score->Channel($args{channel});
-    $score->Octave($args{octave});
-    $score->patch_change( $args{channel}, $args{patch} );
-
-    return $score;
+sub _is_integer0 {
+    croak 'Not greater than or equal to zero'
+        unless defined $_[0] && $_[0] =~ /^\d+$/;
 }
 
-sub _set_chan_patch {
-    my ( $score, $channel, $patch ) = @_;
-
-    $channel //= 0;
-    $patch   //= 0;
-
-    $score->patch_change( $channel, $patch );
-    $score->noop( 'c' . $channel );
-}
-
-sub _invalid_integer {
+sub _is_integer {
     croak 'Invalid integer'
-        unless $_[0] && $_[0] =~ /^\d+$/ && $_[0] > 0;
+        unless defined $_[0] && $_[0] =~ /^\d+$/ && $_[0] > 0;
 }
 
-sub _invalid_list {
+sub _is_list {
     croak 'Invalid list'
         unless ref $_[0] eq 'ARRAY';
 }
 
-sub _invalid_boolean {
+sub _is_boolean {
     croak 'Invalid Boolean'
         unless defined $_[0] && ( $_[0] == 1 || $_[0] == 0 );
 }
@@ -442,25 +417,31 @@ MIDI::Ngram - Find the top repeated note phrases of a MIDI file
 
 =head1 VERSION
 
-version 0.0901
+version 0.1000
 
 =head1 SYNOPSIS
 
   use MIDI::Ngram;
+
   my $mng = MIDI::Ngram->new(
     in_file      => [ 'eg/twinkle_twinkle.mid' ],
     ngram_size   => 3,
     patches      => [qw( 68 69 70 71 72 73 )],
     random_patch => 1,
-    gestalt      => 1,
+    bounds       => 1,
   );
+
   my $analysis = $mng->process;
+
   my $playback = $mng->populate;
+
   $mng->write;
 
 =head1 DESCRIPTION
 
-C<MIDI::Ngram> parses a given list of MIDI files and finds the top repeated note phrases.
+C<MIDI::Ngram> parses a given list of MIDI files, finds the top
+repeated note phrases, outputs the analysis, and renders them to a
+MIDI file if desired.
 
 =head1 ATTRIBUTES
 
@@ -470,76 +451,102 @@ Required.  An ArrayRef of MIDI files to process.
 
 =head2 ngram_size
 
-Ngram phrase size.  Default: 2
+Ngram phrase size.
+
+Default: C<2>
 
 =head2 max_phrases
 
-The maximum number of phrases to play.  Default: 10
+The maximum number of phrases to analyze/play.
+
+Default: C<10>
 
 =head2 bpm
 
-Beats per minute.  Default: 100
+Beats per minute.
+
+Default: C<100>
 
 =head2 durations
 
-The note durations to choose from (at random).  Default: [qn tqn]
+The note durations to choose from (at random).
+
+Default: C<[hn qn en]>
 
 =head2 patches
 
 The patches to choose from (at random) if given the B<random_patch> option.
-Otherwise 0 (piano) is used.  Default: [0 .. 127]
+
+Default: C<[0 .. 127]>
 
 =head2 out_file
 
-MIDI output file.  Default: midi-ngram.mid
+MIDI output file.
+
+Default: C<midi-ngram.mid>
 
 =head2 pause_duration
 
-Insert a rest of the given duration after each phrase.  Default: '' (no resting)
+Insert a rest of the given duration after each phrase.
+
+Default: C<''> (no resting)
 
 =head2 analyze
 
 ArrayRef of the channels to analyze.  If not given, all channels are analyzed.
 
+Default: C<[]>
+
 =head2 loop
 
-The number of times to choose a weighted phrase.  * Only works with the
-B<weight> option.  Default: 4
+The number of times to choose a weighted phrase.  * This only works
+with the B<weight> option.
+
+Default: C<4>
 
 =head2 weight
 
-Boolean.  Play phrases by their ngram repetition occurrence.  Default: 0
+Boolean.  Play phrases by their ngram repetition occurrence.
+
+Default: C<0>
 
 =head2 random_patch
 
 Boolean.  Choose a random patch from B<patches> for each channel.
-Default: 0 (piano)
+
+Default: C<0> (meaning "use the piano patch")
 
 =head2 shuffle_phrases
 
-Boolean.  Shuffle the non-weighted phrases before playing them.  Default: 0
+Boolean.  Shuffle the non-weighted phrases before playing them.
+
+Default: C<0>
 
 =head2 single_phrases
 
-Boolean.  Allow single occurrence ngrams.  Default: 0
+Boolean.  Allow single occurrence ngrams.
+
+Default: C<0>
 
 =head2 one_channel
 
-Boolean.  Accumulate phrases into a single list.  Default: 0
+Boolean.  Accumulate phrases into a single list.
 
-=head2 gestalt
+Default: C<0>
+
+=head2 bounds
 
 Boolean.  Include pitch range in the analysis.
 
+Default: C<0>
+
 =head2 score
 
-The MIDI score object.  Constructed at runtime.  Constructor argument if given
-will be ignored.
+The L<MIDI::Simple/"MAIN-ROUTINES"> score object.  Constructed at runtime.
 
 =head2 notes
 
-The bucket of ngrams.  Constructed at runtime.  Constructor argument if given
-will be ignored.
+The hash-reference bucket of ngrams.  Constructed at runtime.
 
 =head1 METHODS
 
@@ -595,7 +602,7 @@ Gene Boggs <gene@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Gene Boggs.
+This software is copyright (c) 2020 by Gene Boggs.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
